@@ -140,7 +140,7 @@ main(int argc, char **argv)
 		}
 	}
 
-	if (portdesc == NULL) {
+	if (portdesc == NULL || portdesc[0] == '\0') {
 		portdesc = getenv("ALSA_OUTPUT_PORTS");
 		/* Try the old name for the environment variable */
 		if (portdesc == NULL)
@@ -307,7 +307,6 @@ showlist()
 	snd_seq_client_info_t *cinfo;
 	snd_seq_port_info_t *pinfo;
 	int  client;
-	int  port;
 	int  err;
 	snd_seq_t *handle;
 
