@@ -1,7 +1,7 @@
 /*
  * File: seqlib.m - Interface to the alsa sequencer library.
  * 
- * Copyright (C) 1999 Steve Ratcliffe
+ * Copyright (C) 1999-2003 Steve Ratcliffe
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -25,6 +25,8 @@
 #include "seqlib.h"
 #include "seqpriv.h"
 
+static void set_channel(snd_seq_event_t *ep, int chan);
+
 /*
  * Create a main client for an application. A queue is
  * allocated and a client is created.
@@ -77,16 +79,27 @@ seq_new_client(seq_context_t *ctxp)
  * link all together so can all be closed at once.
  */
 void 
-seq_free_context(seq_context_t *cxtp)
+seq_free_context(seq_context_t *ctxp)
 {
 
-	if (cxtp->main == cxtp) {
+	if (ctxp->main == ctxp) {
 		/* This is the main context */
-		snd_seq_free_queue(cxtp->handle, cxtp->queue);
-		snd_seq_close(cxtp->handle);
+		snd_seq_event_t ev;
+		unsigned long t;
+
+		snd_seq_drop_output(ctxp->handle);
+
+		t = 0;
+		seq_midi_event_init(ctxp, &ev, t, 0);
+		seq_midi_control(ctxp, &ev, 0, MIDI_CTL_ALL_SOUNDS_OFF, 0);
+		seq_send_to_all(ctxp, &ev);
+		snd_seq_drain_output(ctxp->handle);
+
+		snd_seq_free_queue(ctxp->handle, ctxp->queue);
+		snd_seq_close(ctxp->handle);
 	}
 
-	g_free(cxtp);
+	g_free(ctxp);
 }
 
 /*
@@ -116,7 +129,7 @@ seq_new_port(seq_context_t *ctxp)
 }
 
 void 
-seq_destroy_port(seq_context_t *cxtp, int port)
+seq_destroy_port(seq_context_t *ctxp, int port)
 {
 }
 
@@ -210,6 +223,66 @@ seq_sendto(seq_context_t *ctxp, snd_seq_event_t *ev, int client, int port)
 	return 0;
 }
 
+/*
+ * seq_send_to_all:
+ * Send the event to all the connected devices across all
+ * possible channels. The messages are sent to the blocking
+ * control port and so should not have timestamps in the
+ * future. This function is intended for all-sounds-off type
+ * controls etc.
+ *  
+ *  Arguments:
+ * 	ctxp: Client context
+ * 	ep: Event prototype to be sent
+ */
+int 
+seq_send_to_all(seq_context_t *ctxp, snd_seq_event_t *ep)
+{
+	int  dev;
+	int  chan;
+	snd_seq_addr_t *addr;
+
+	for (dev = 0; ; dev++) {
+		addr = seq_dev_addr(ctxp, dev);
+		if (addr == NULL)
+			break; /* No more */
+
+		ep->queue = ctxp->queue;
+		ep->dest = *addr;
+		for (chan = 0; chan < 16; chan++) {
+			set_channel(ep, chan);
+			(void) seq_write(ctxp, ep);
+		}
+	}
+
+	return 0;
+}
+
+/*
+ * seq_dev_addr:
+ * Return the address corresponding to the specified logical
+ * device.
+ *  
+ * 	Arguments:
+ * 	dev: Device to get the address of.
+ * 	inout: Input or output device
+ */
+snd_seq_addr_t *
+seq_dev_addr(seq_context_t *ctxp, int dev)
+{
+	GArray *list;
+	snd_seq_addr_t *addr;
+
+	list = ctxp->destlist;
+
+	if (dev >= (int)list->len)
+		return NULL;
+
+	addr = &g_array_index(list, snd_seq_addr_t, dev);
+
+	return addr;
+}
+
 /*
  * Start the timer. (What about timers other than the system
  * one?)
@@ -277,3 +350,29 @@ seq_handle(seq_context_t *ctxp)
 	return ctxp->handle;
 }
 
+static void 
+set_channel(snd_seq_event_t *ep, int chan)
+{
+
+	switch (ep->type) {
+	case SND_SEQ_EVENT_NOTE:
+	case SND_SEQ_EVENT_NOTEON:
+	case SND_SEQ_EVENT_NOTEOFF:
+		ep->data.note.channel = chan;
+		break;
+	case SND_SEQ_EVENT_KEYPRESS:
+	case SND_SEQ_EVENT_PGMCHANGE:
+	case SND_SEQ_EVENT_CHANPRESS:
+	case SND_SEQ_EVENT_PITCHBEND:
+	case SND_SEQ_EVENT_CONTROL14:
+	case SND_SEQ_EVENT_NONREGPARAM:
+	case SND_SEQ_EVENT_REGPARAM:
+	case SND_SEQ_EVENT_CONTROLLER:
+		ep->data.control.channel = chan;
+		break;
+	default:
+		if (snd_seq_ev_is_channel_type(ep))
+			g_warning("Missed a case in set_channel");
+		break;
+	}
+}
