@@ -1,7 +1,7 @@
 /*
  * File: seqlib.h
  * 
- * Copyright (C) 1999 Steve Ratcliffe
+ * Copyright (C) 1999-2003 Steve Ratcliffe
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -40,6 +40,8 @@ int seq_init_tempo(seq_context_t *ctxp, int resolution, int tempo,
         int realtime);
 void seq_set_queue(seq_context_t *ctxp, int q);
 int seq_sendto(seq_context_t *ctxp, snd_seq_event_t *ev, int client, int port);
+int seq_send_to_all(seq_context_t *ctxp, snd_seq_event_t *ep);
+snd_seq_addr_t * seq_dev_addr(seq_context_t *ctxp, int dev);
 void seq_start_timer(seq_context_t *ctxp);
 void seq_stop_timer(seq_context_t *ctxp);
 void seq_control_timer(seq_context_t *ctxp, int onoff);
